using System;
using System.ComponentModel;
using System.Globalization;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections;
using System.Reflection;
using System.Xml.Serialization;
using System.Drawing;

using Syncfusion.Windows.Forms.Diagram;
using Symcon.LightRidge.BusinessObjects.TypeConverters;
using Symcon.LightRidge.DashBoardControlsLibrary;
using PerpetuumSoft.Instrumentation.Model;
using PerpetuumSoft.Instrumentation.Windows.Forms;

namespace Symcon.Lightridge.LRRUI.Controls
{
    [Serializable()]
    public class DashBoardControlProperties : Node
    {
        [NonSerialized]
        private object _controlObject;
        
        public object ControlObject
        {
            get { return _controlObject; }
            set { _controlObject = value; }
        }
	

        #region Constructor
        /// <summary>
        /// Default constructor.
        /// </summary>
        public DashBoardControlProperties(Control dashBoardControl):base()
        {
            _controlObject = dashBoardControl;
            SetPropertyTable(dashBoardControl);
            this.ID = Guid.NewGuid().ToString();
            //CreateLabels();
            ArryIndexedProps = new ArrayList();


            for (int i = 0; i < 4; i++)
            {
                ArryIndexedProps.Add("Null");
            }
            //SymbolsUtils.CreateLabels(this);
        }
        /// <summary>
        /// Copy constructor.
        /// </summary>
        /// <param name="src"></param>
        public DashBoardControlProperties(DashBoardControlProperties src)
            : base(src)
        {

        }


        public DashBoardControlProperties()
        {

        }
        /// <summary>
        /// Serialization constructor for the MySymbol class.
        /// </summary>
        /// <param name="info">Serialization state information</param>
        /// <param name="context">Streaming context information</param>
        protected DashBoardControlProperties(SerializationInfo info, StreamingContext context): base(info, context)
            
        {
            //this.indicatorWidget1.InstrumentationStream = info.GetString("InstruStream");
            //this._controlObject = info.GetValue("ControlObject", typeof(object));
            //this.PT = (PropertyTable)info.GetValue("PropertyTable", typeof(PropertyTable));
            //info.AddValue("InstruStream", this.indicatorWidget1.InstrumentationStream);
            //info.AddValue("ControlObject", this._controlObject);
            // The Serialization constructor is invoked during deserialization or during a drag & drop operation.
            // If the MySymbol type has serializable members, then initialize them with the serialized data 
            // obtained from the SerializationInfo param
        }

        // Override SymbolBase.GetObjectData() and populate the SerializationInfo param 
        // with the data (if any) that belongs to the MySymbol type . This data will be 
        // serialized as a part of the Symbol object.




        #endregion

        #region	Public Properties

        /// <summary>
        /// Link to Navigate to when symbol is double-clicked
        /// </summary>
        [Browsable(true), TypeConverter(typeof(LinkTypeConverter)), ReadOnly(false)]
        public Symcon.LightRidge.BusinessObjects.GraphicalLink Link
        {
            get
            {
                object value = this.GetPropertyValue("Link");
                if (value != null && value.GetType() != typeof(string))
                {

                    return (Symcon.LightRidge.BusinessObjects.GraphicalLink)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("Link", value);
                ((PathNode)((Node)this)).Labels["Link"].Text = Link.Name;
            }
        }



        [Browsable(false), ReadOnly(false)]
        public Color Color
        {
            get
            {

                return this.FillStyle.Color;
                //				object value = this.GetPropertyValue("FillStyle.Color");
                //				if (value != null)
                //				{
                //					return (Color) value;
                //				}
                //				return base.FillStyle.Color;
            }
            set
            {
                //	this.LineStyle.LineColor = value;;
                this.FillStyle.Color = value;
                //this.SetPropertyValue("FillStyle.Color", value);
            }
        }


        [Browsable(false), ReadOnly(false)]
        public Color BorderColor
        {
            get
            {
                return this.LineStyle.LineColor;
            }
            set
            {
                this.LineStyle.LineColor = value;
            }
        }


        [Browsable(true), ReadOnly(false)]
        public new LineStyle LineStyle
        {
            get
            {
                return base.LineStyle;
            }
        }

        [Browsable(true), ReadOnly(false)]
        public new FillStyle FillStyle
        {
            get
            {
                return ((FilledPath)((Node)this)).FillStyle;
            }
        }




        [Browsable(true), ReadOnly(false)]
        public float BorderWidth
        {
            get
            {
                return this.LineStyle.LineWidth;
            }
            set
            {
                this.LineStyle.LineWidth = value;
            }
        }

        [Browsable(true), ReadOnly(false)]
        public bool LockAspectRatio
        {
            get
            {
                object value = this.GetPropertyValue("LockAspectRatio");
                if (value != null)
                {
                    return (bool)value;
                }
                return true;
            }
            set
            {
                this.SetPropertyValue("LockAspectRatio", value);
            }
        }




        [Browsable(true), ReadOnly(false)]
        public int Transparency
        {
            get
            {
                return this.FillStyle.ColorAlphaFactor;
            }
            set
            {
                this.FillStyle.ColorAlphaFactor = value;
            }
        }


        /// <summary>
        /// Link Id
        /// </summary>
        [Browsable(false), ReadOnly(false)]
        public string LinkID
        {
            get
            {
                object value = this.GetPropertyValue("LinkID");
                if (value != null)
                {
                    return (string)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("LinkID", value);
            }
        }


        /// <summary>
        /// The Type of Master Plant Item
        /// Associated with this symbol
        /// </summary>
        [Browsable(true), TypeConverter(typeof(SymbolTypeConverter)),
        CategoryAttribute("Binding Information")]
        public Symcon.LightRidge.BusinessObjects.MasterPlantItem MasterPlantItem
        {
            get
            {
                object value = this.GetPropertyValue("MasterPlantItem");
                if (value != null)
                {
                    return (Symcon.LightRidge.BusinessObjects.MasterPlantItem)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("MasterPlantItem", value);
                this.SetPropertyValue("PlantItem", null);
            }
        }


        /// <summary>
        /// The Type of Master Plant Item
        /// Associated with this symbol
        /// </summary>
        [Browsable(false), TypeConverter(typeof(SymbolTypeConverter)),
        CategoryAttribute("Binding Information")]
        public string ItemType
        {
            get
            {
                object value = this.GetPropertyValue("ItemType");
                if (value != null)
                {
                    return (string)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("ItemType", value);
                this.SetPropertyValue("SiteItem", null);
            }
        }


        /// <summary>
        /// The Type of Plant Item
        /// Associated with this symbol
        /// </summary>
        [Browsable(true), TypeConverter(typeof(ObjectNameConverter)),
        CategoryAttribute("Binding Information")]
        public Symcon.LightRidge.BusinessObjects.SiteItem PlantItem
        {
            get
            {
                object value = this.GetPropertyValue("PlantItem");
                if (value != null)
                {
                    return (Symcon.LightRidge.BusinessObjects.SiteItem)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("PlantItem", value);
            }
        }



        /// <summary>
        /// The Type of Plant Item
        /// Associated with this symbol
        /// </summary>
        [Browsable(false), TypeConverter(typeof(ObjectNameConverter)),
        CategoryAttribute("Binding Information")]
        public string SiteItem
        {
            get
            {
                object value = this.GetPropertyValue("SiteItem");
                if (value != null)
                {
                    return (string)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("SiteItem", value);
            }
        }


        ///// <summary>
        ///// The Width of Symbol
        ///// </summary>
        //[Browsable(true)]
        //public float Length
        //{
        //    get
        //    {
        //        return (float)base.Width;
        //    }
        //    set
        //    {
        //        base.Width = value;

        //    }
        //}

        ///// <summary>
        ///// The Width of Symbol
        ///// </summary>
        //[Browsable(true)]
        //public new float Height
        //{
        //    get
        //    {
        //        return (float)base.Height;
        //    }
        //    set
        //    {
        //        base.Height = value;
        //    }
        //}



        ///// <summary>
        ///// The X Cordinate of Symbol w.r.t the Canvas
        ///// </summary>
        //[Browsable(true)]
        //public new float X
        //{
        //    get
        //    {
        //        return base.X;
        //    }
        //    set
        //    {
        //        base.X = value;
        //    }
        //}


        ///// <summary>
        ///// The Y Cordinate of Symbol w.r.t the Canvas
        ///// </summary>
        //[Browsable(true)]
        //public new float Y
        //{
        //    get
        //    {
        //        return base.Y;
        //    }
        //    set
        //    {
        //        base.Y = value;
        //    }
        //}



        /// <summary>
        /// Set of Labels 
        /// </summary>
        [Browsable(false)]
        public LabelCollection BoundProperties
        {
            get
            {
                return ((PathNode)((Node)this)).Labels;
            }
        }


        /// <summary>
        /// PropertyTable Associated With the symbol
        /// </summary>
        [Browsable(false)][XmlIgnore()]
        public PropertyTable PT
        {
            get
            {
                object value = this.GetPropertyValue("PT");
                if (value != null)
                {
                    return (PropertyTable)value;
                }
                PT = new PropertyTable();
                return null;
            }
            set
            {
                try
                {
                    this.SetPropertyValue("PT", value);
                }
                catch (Exception ex)
                {
                    Debug.Assert(false, ex.Message);
                }
            }
        }

        [Browsable(false), ReadOnly(false)]
        public string SymbolName
        {
            get
            {
                object value = this.GetPropertyValue("SymbolName");
                if (value != null)
                {
                    return (string)value;
                }
                return "Test";
            }
            set
            {
                try
                {
                    this.SetPropertyValue("SymbolName", value);
                }
                catch (Exception ex)
                {
                    Debug.Assert(false, ex.Message);
                }
            }
        }

        [Browsable(false), ReadOnly(false)]
        public string ID
        {
            get
            {
                object value = this.GetPropertyValue("ID");
                if (value != null)
                {
                    return (string)value;
                }
                return null;
            }
            set
            {
                try
                {
                    this.SetPropertyValue("ID", value);
                }
                catch (Exception ex)
                {
                    Debug.Assert(false, ex.Message);
                }
            }
        }

        [Browsable(false), ReadOnly(false)]
        public ArrayList ArryIndexedProps
        {
            get
            {
                object value = this.GetPropertyValue("ArryIndexedProps");
                if (value != null)
                {
                    return (ArrayList)value;
                }
                return null;
            }
            set
            {
                this.SetPropertyValue("ArryIndexedProps", value);                
            }

        }

        #endregion

        #region Custom Properties
        private void SetDialControlProperties(Control dashBoardControl)
        {
            Symcon.LightRidge.DashBoardControlsLibrary.PropertyBag.PropertyTable pt = dashBoardControl.GetType().GetProperty("PT").GetValue(dashBoardControl, null) as Symcon.LightRidge.DashBoardControlsLibrary.PropertyBag.PropertyTable;
            foreach (Symcon.LightRidge.DashBoardControlsLibrary.PropertyBag.PropertySpec ps in pt.Properties)
            {
                this.PT.Properties.Add(new PropertySpec(ps.Name, ps.TypeName, ps.Category, ps.Description, ps.DefaultValue, ps.EditorTypeName, ps.ConverterTypeName, ps.ObjectRef));
            }
        }
        #endregion

        #region Fields
        private IndicatorWidget indicatorWidget1;
        #endregion

        public void SetPropertyTable(Control dashBoardControl)
        {
            if (PT == null)
            {
                try
                {
                    SetDialControlProperties(dashBoardControl);
                    PropertyInfo[] properties = this.GetType().GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
                    foreach (PropertyInfo property in properties)
                    {
                        BrowsableAttribute[] attr = property.GetCustomAttributes(typeof(BrowsableAttribute), false) as BrowsableAttribute[];
                        if (attr.Length > 0)
                            if (((BrowsableAttribute)attr[0]).Browsable == false)
                                continue;
                        string propname = property.Name;
                        if (property.Name == "MasterPlantItem")
                        {
                            //PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Select the Type of Symbol" ,this.GetPropertyValue(property.Name),"",typeof(SymbolTypeConverter),this));
                            PT.Properties.Add(new PropertySpec(property.Name, property.PropertyType, "Binding Information", "Lists of Master Plant Item", this.GetPropertyValue(property.Name), "", typeof(SymbolTypeConverter), this));
                        }
                        else if (property.Name == "PlantItem")
                        {
                            //PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Lists the Bound objects of selected type" ,this.GetPropertyValue(property.Name),"",typeof(ObjectNameConverter),this));
                            PT.Properties.Add(new PropertySpec(property.Name, property.PropertyType, "Binding Information", "Lists the Plant Item objects of Selected Master Plant Item", this.GetPropertyValue(property.Name), "", typeof(ObjectNameConverter), this));
                            //siteitemps = new PropertySpec("PlantItem",typeof(SiteItem),"Binding Information","Lists the Plant Item objects of Selected Master Plant Item" ,si,"",typeof(Symcon.LightRidge.BusinessObjects.TypeConverters.ObjectNameConverter),sm);
                        }
                        else if (property.Name == "Link")
                        {
                            PT.Properties.Add(new PropertySpec(property.Name, property.PropertyType, "Link", "Select the Link for the Symbol", this.GetPropertyValue(property.Name), "", typeof(LinkTypeConverter), this));
                        }
                        else
                        {
                            PT.Properties.Add(new PropertySpec(property.Name, property.PropertyType, "General	Properties", "Sets the " + property.Name + " property of selected symbol", this.GetPropertyValue(property.Name), "", "", this));
                        }
                    }
                }
                catch (Exception ex)
                {
                    Debug.Assert(false, ex.Message);
                }
            }
        }

        
    }
}
