using System;

using System.Drawing;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Diagram.Controls;
using System.Runtime.Serialization;
//using Symcon.LightRidge.DashBoardControlsLibrary.Controls;
//using Symcon.LightRidge.DashBoardControlsLibrary.PropertyBag;
//using Symcon.LightRidge.DashBoardControlsLibrary.TypeConverters;

using Symcon.LightRidge.BusinessObjects;

using Symcon.LightRidge.BusinessObjects.Utilities;
using Symcon.LightRidge.BusinessObjects.TypeEditors;
using Symcon.LightRidge.BusinessObjects.TypeConverters;
using Symcon.LightRidge.BusinessObjects.Utilities;
using System.Collections;
using System.ComponentModel;
using System.Reflection;
using System.Diagnostics;
using System.Drawing.Drawing2D;
using PerpetuumSoft.Framework.Model;


namespace Symcon.Lightridge.LRRUI.Controls 
{
	[Serializable()]
	public class GridCell : Syncfusion.Windows.Forms.Diagram.Node,ISerializable
	{
		#region Fields

		public event NodeMouseEventHandler GridCellClick;
		public event NodeMouseEventHandler GridCellDoubleClick;
		public event EventHandler RowSpanChanged;
		public event EventHandler ColumnSpanChanged;


		private float _height;
		private float _width;
		private int _rowIndex;
		private int _colIndex;
		private string _id = null;
		private string _propertyID= null;
		private string _modeID = null;
		private Type _type;
		private bool _isBinded;
		[NonSerialized()]
		private object _tag;
		private string _uomID;
		private ErrorCode _labelColorCode;
	//	private DynamicGridcontainer _parentGrid;
		private string _uomTypeID = null;
		private string _description = "";
		
		private int _oldColSpan = 1;
		private int _oldRowSpan = 1;
		private int _colSpan =1;
		private int _rowSpan=1;

		#endregion
	
		#region Constructors

		public GridCell()
		{
			SetPropertyTable();
			this.HorizontalAlignment = System.Drawing.StringAlignment.Center;
			this.VerticalAlignment = System.Drawing.StringAlignment.Center;
            this.FontFamily = "Arial";
			//_parentGrid = this.Parent as DynamicGridcontainer;
			//this.FontSize = 10;
		}

	
		public GridCell(GridCell src) :base(Syncfusion.Windows.Forms.Diagram.Symbol)
		{
			this.PT =null;
			SetPropertyTable();
			this.ColumnIndex = src.ColumnIndex;
			this.RowIndex = src.RowIndex;
			this.RowSpan = src.RowSpan;
			this.ColSpan = src.ColSpan;

			this.OldColumnSpan = src.OldColumnSpan;
			this.OldRowSpan = src.OldRowSpan;
			//this.OffsetX = src.OffsetX;
			//this.OffsetY = src.OffsetY;
	
			this.FontFamily=src.FontFamily;
			this.FontSize=src.FontSize;
			this.FontStyle.Family = src.FontStyle.Family;
			this.FontStyle.Bold = src.FontStyle.Bold;
			//this.FontStyle.Italic = src.FontStyle.Italic;
			//this.FontStyle.PointSize = src.FontStyle.PointSize;
			//this.FontStyle.Size = src.FontStyle.Size;
			//this.FontStyle.Strikeout = src.FontStyle.Strikeout;
			//this.FontStyle.Style = src.FontStyle.Style;

			//this.FillStyle.AlphaFactor = src.FillStyle.AlphaFactor;
			this.FillStyle.Color = src.FillStyle.Color;
			this.FillStyle.GradientAngle = src.FillStyle.GradientAngle;
			//this.FillStyle.GradientBounds = src.FillStyle.GradientBounds;
			//this.FillStyle.GradientStartColor = src.FillStyle.GradientStartColor;
			this.FillStyle.Texture = src.FillStyle.Texture;
			this.FillStyle.TextureWrapMode = src.FillStyle.TextureWrapMode;
			
		}
        //}


		/// <summary>
		/// Serialization constructor for the MySymbol class.
		/// </summary>
		/// <param name="info">Serialization state information</param>
		/// <param name="context">Streaming context information</param>
		protected GridCell(SerializationInfo info, StreamingContext context) : base(info, context)
		{
			// The Serialization constructor is invoked during deserialization or during a drag & drop operation.
			// If the MySymbol type has serializable members, then initialize them with the serialized data 
			// obtained from the SerializationInfo param
			try
			{
				this._id = (string) info.GetString("_id");
				this._modeID = (string) info.GetString("_modeID");
				this._uomID = (string) info.GetString("_uomID");
				this._propertyID = (string) info.GetString("_propertyID");
				this._isBinded  = (bool) info.GetBoolean("_isBinded");
				this._rowIndex = (int) info.GetInt32("_rowIndex");
				this._colIndex = (int) info.GetInt32("_colIndex");
				
				this._width = (float) info.GetSingle("_width");
				this._height= (float) info.GetSingle("_height");
				this._type= (System.Type) info.GetValue("_type",typeof(System.Type));
				this._labelColorCode = (ErrorCode) info.GetValue("_labelColorCode",typeof(ErrorCode));
				this._uomTypeID = (string) info.GetString("_uomTypeID");
				//this._parentGrid = this.Parent as DynamicGridcontainer;
				this._description= (string) info.GetString("_description");

				this._oldColSpan=(int) info.GetInt32("_oldColSpan");
				this._oldRowSpan=(int)info.GetInt32("_oldRowSpan");
                this._colSpan = (int)info.GetInt32("_colSpan");
                this._rowSpan = (int)info.GetInt32("_rowSpan");
				
			}
			catch
			{


			}
		}

		
		#endregion
		
		#region Properties

		/// <summary>
		/// The Type of Master Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(true),TypeConverter(typeof(SymbolTypeConverter)), 
		CategoryAttribute("Binding Information")]
		public Symcon.LightRidge.BusinessObjects.MasterPlantItem MasterPlantItem
		{
			get
			{
				object value = this.GetPropertyValue("MasterPlantItem");
				if (value != null)
				{
					return (Symcon.LightRidge.BusinessObjects.MasterPlantItem) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("MasterPlantItem", value);
				this.SetPropertyValue("PlantItem", null);
			}
		}


		/// <summary>
		/// The Type of Master Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(false),TypeConverter(typeof(SymbolTypeConverter)), 
		CategoryAttribute("Binding Information")]
		public string ItemType
		{
			get
			{
				object value = this.GetPropertyValue("ItemType");
				if (value != null)
				{
					return (string) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("ItemType", value);
				this.SetPropertyValue("SiteItem", null);
			}
		}


		/// <summary>
		/// The Type of Plant Item
		/// Associated with this symbol
		/// </summary>
		[Browsable(true),TypeConverter(typeof(ObjectNameConverter)), 
		CategoryAttribute("Binding Information")]
		public Symcon.LightRidge.BusinessObjects.SiteItem PlantItem
		{
			get
			{
				object value = this.GetPropertyValue("PlantItem");
				if (value != null)
				{
					return (Symcon.LightRidge.BusinessObjects.SiteItem) value;
				}
				return null;
			}
			set
			{
				this.SetPropertyValue("PlantItem", value);
			}
		}

	
			/// <summary>
		/// The Column span
		/// Associated with this cell
		/// </summary>
		[Browsable(true), 
		CategoryAttribute("Merge Information")]
		public int ColSpan
		{
			get { return _colSpan; }
			set {
				if (_colSpan != value)
				{
					_oldColSpan = _colSpan;
					_colSpan = value;
					if (ColumnSpanChanged != null)
					{
						ColumnSpanChanged(this,null);
					}
				}	
				
			}
		}
		

		internal void ResetColSpan()
		{
			_colSpan = _oldColSpan;
		}

		internal void ResetRowSpan()
		{
			_rowSpan = _oldRowSpan;
		}

		/// <summary>
		/// The Row span
		/// Associated with this cell
		/// </summary>
		/// 
		[Browsable(true), 
		CategoryAttribute("Merge Information")]
		public int RowSpan
		{
			get { return _rowSpan; }
			set
			{
				if (_rowSpan != value)
				{
					_oldRowSpan = _rowSpan;
						_rowSpan = value;
						if (RowSpanChanged != null)
						{
							RowSpanChanged(this,null);
						}
				}
			
			}
		}


		

		internal int OldColumnSpan
		{
			get { return _oldColSpan; }
			set { _oldColSpan = value; }
		}
	
		
		internal int OldRowSpan
		{
			get { return _oldRowSpan; }
				set { _oldRowSpan = value; }
		}
	

		/// <summary>
		/// The Column Index
		/// Associated with this cell
		/// </summary>
		[Browsable(false), 
		CategoryAttribute("Index Information")]
		public int RowIndex
		{
			get
			{
				return _rowIndex;
			}
			set
			{
				_rowIndex = value;
			}
		}


		/// <summary>
		/// The Row Index
		/// Associated with this Cell
		/// </summary>
		[Browsable(false), 
		CategoryAttribute("Index Information")]
		public int ColumnIndex
		{
			get
			{
				return _colIndex;
			}
			set
			{
				_colIndex = value;
			}
		}


		/// <summary>
		/// The Row Index
		/// Associated with this Cell
		/// </summary>
		[Browsable(false), 
		CategoryAttribute("Index Information")]
		public object Tag
		{
			get
			{
				return _tag;
			}
			set
			{
				_tag = value;
			}
		}

	
		[Browsable(false)]
		public ErrorCode LabelColorCode
		{
			get
			{
				return _labelColorCode;
			}
			set
			{
				_labelColorCode = value;

			}
		}

		
		/// <summary>
		/// PropertyTable Associated With the symbol
		/// </summary>
		[Browsable(false)]
		public PropertyTable PT
		{
			get
			{
				object value = this.GetPropertyValue("PT");
				if (value != null)
				{
					return (PropertyTable) value;
				}
				PT = new PropertyTable();
				return null;
			}
			set
			{
				try
				{
					this.SetPropertyValue("PT", value);
				}
				catch(Exception ex)
				{
					System.Diagnostics.Debug.Assert(false, ex.Message);
				}
			}
		}



		[Browsable(false)]
		public string ID
		{
			get{return _id;}
			set
			{
				_id = value;
			}
		}


		[Browsable(false)]
		public string PropertyID
		{
			get{return _propertyID ;}
			set
			{
				_propertyID = value;
			}
		}

		[Browsable(false)]
		public string ModeID
		{
			get{return _modeID;}
			set
			{
				_modeID = value;
			}
		}


		[Browsable(false)]
		public bool IsBinded
		{
			get
			{
				return _isBinded;
			}
			set
			{
				_isBinded = value;
				if(value)
				{
					Attribute[]	attrib = new Attribute[1];
					BrowsableAttribute browsableAttrib= new BrowsableAttribute(false);
					attrib[0] = browsableAttrib;
					PT.Properties[PT.Properties.IndexOf("Text")].AttributesNew = attrib;
					PT.Properties[PT.Properties.IndexOf("BackgroundStyle")].AttributesNew = attrib;
					PT.Properties[PT.Properties.IndexOf("FillStyle")].AttributesNew = attrib;
					PT.Properties[PT.Properties.IndexOf("ShowBorder")].AttributesNew = attrib;
				}
				else
				{
					Attribute[]	attrib = new Attribute[1];
					BrowsableAttribute browsableAttrib= new BrowsableAttribute(true);
					attrib[0] = browsableAttrib;
					PT.Properties[PT.Properties.IndexOf("Text")].AttributesNew = attrib;
					PT.Properties[PT.Properties.IndexOf("BackgroundStyle")].AttributesNew = attrib;
					PT.Properties[PT.Properties.IndexOf("FillStyle")].AttributesNew = attrib;
					PT.Properties[PT.Properties.IndexOf("ShowBorder")].AttributesNew = attrib;
					if(this.ParentGrid != null)
					{
						//this.BorderStyle.BorderColor = this.ParentGrid.BorderColor;
						//this.BorderStyle.ShowBorder = this.ParentGrid.ShowGridLines;
						this.BorderStyle.BorderWidth = this.ParentGrid.BorderWidth;
					}
				}
			}
		}

		[Browsable(false)]
		public Type Type
		{
			get
			{
				return _type;
			}
			set
			{
				_type = value;
			}
		}


		public new string Text
		{
			get
			{
				//base.Text = "CI =" + this.ColumnIndex + " RI =" + this.RowIndex + "CS =" + this.ColSpan+ " RS =" + this.RowSpan;// +
				return base.Text;
			}
			set
			{
				base.Text = value;
			}
		}


		[Browsable(false)]
		public string Description
		{
			get
			{
				return 
					_description;
			}
			set
			{
				_description = value;
			}
		}

		
		[Browsable(true),  ReadOnly(false), TypeConverter(typeof(FontTypeConverter))]		
		public string FontFamily
		{
			get
			{
				return base.FontStyle.Family;
			}
			set
			{
				base.FontStyle.Family= value;
			}
		}


		[Browsable(true)]
		public float FontSize
		{
			get
			{
				return base.FontStyle.Size;
			}
			set
			{
				base.FontStyle.Size= value;
			}
		}


		[Browsable(true)]
		public System.Drawing.FontStyle TextStyle
		{
			get
			{
				return base.FontStyle.Style;
			}
			set
			{
				base.FontStyle.Style= value;
			}
		}


		[Browsable(true)]
		public new Syncfusion.Windows.Forms.Diagram.FillStyle FillStyle
		{
			get
			{
				return base.FillStyle;
			}
		}


		[Browsable(true)]
		public new Syncfusion.Windows.Forms.Diagram.BackgroundStyle BackgroundStyle
		{
			get
			{
				return base.BackgroundStyle;
			}
		}


		[Browsable(true)]
		public bool ShowBorder
		{
			get
			{
				return base.BorderStyle.ShowBorder;
			}
			set
			{
				base.BorderStyle.ShowBorder = value;
			}
		}

		
		[Browsable(true)]
		public new System.Drawing.StringAlignment HorizontalAlignment
		{
			get
			{
				return base.HorizontalAlignment;
			}
			set
			{
				base.HorizontalAlignment = value;
			}
		}

		
		[Browsable(true)]
		public new System.Drawing.StringAlignment VerticalAlignment
		{
			get
			{
				return base.VerticalAlignment;
			}
			set
			{
				base.VerticalAlignment = value;
			}
		}


		[Browsable(false)]
		public string UIUOMID
		{
			get
			{
				return _uomID ;
			}
			set
			{
				_uomID = value;
			}
		}




		[Browsable(false)]
		public string UOMTypeID
		{
			get
			{
				return _uomTypeID ;
			}
			set
			{
				_uomTypeID = value;
			}
		}


		[Browsable(false),System.Xml.Serialization.XmlIgnore()]
		private DynamicGridcontainer ParentGrid
		{
			get
			{
				return this.Parent as DynamicGridcontainer;
			}
		}

        
        [Browsable(true),Category("Formatting")]
        public new bool WrapText 
        {
            get { return base.WrapText; }
            set { base.WrapText = value; }
        }

        [Browsable(true), Category("Formatting")]
        public new bool NoClip
        {
            get { return base.NoClip; }
            set { base.NoClip = value; }
        }

        [Browsable(true), Category("Formatting")]
        public new bool LineLimit
        {
            get { return base.LineLimit; }
            set { base.LineLimit = value; }
        }


        [Browsable(true), Category("Formatting")]
        public new bool FitBlackBox
        {
            get { return base.FitBlackBox; }
            set { base.FitBlackBox = value; }
        }

        [Browsable(true), Category("Formatting")]
        public new bool DirectionRightToLeft
        {
            get { return base.DirectionRightToLeft; }
            set { base.DirectionRightToLeft = value; }
        }

        [Browsable(true), Category("Formatting")]
        public new bool DirectionVertical
        {
            get { return base.DirectionVertical; }
            set { base.DirectionVertical = value; }
        }

        [Browsable(true), Category("Formatting")]
        public new bool MeasureTrailingSpaces
        {
            get { return base.MeasureTrailingSpaces; }
            set { base.MeasureTrailingSpaces = value; }
        }

	

	
		#endregion
		
		#region Overrides
 


		public override object Clone()
		{
			return new GridCell(this);
		}

		
		//protected override void OnClick(NodeMouseEventArgs evtArgs)
		//{
			//base.OnClick (evtArgs);
			//if(GridCellClick != null)
				//GridCellClick(this,evtArgs);
		//}

		
		/*protected override void OnPropertyChanged(PropertyEventArgs evtArgs)
		{
			if(evtArgs.PropertyName == "Text")
			{
				base.OnPropertyChanged (evtArgs);
				this.Width = _width;
				this.Height = _height;
			}
		}*/


		/*protected override void OnPropertyChanging(PropertyEventArgs evtArgs)
		{
			if(evtArgs.PropertyName == "Text")
			{
				_width = this.Width;
				_height = this.Height;
				base.OnPropertyChanged (evtArgs);
			}
		}*/

		/*protected override void OnDoubleClick(NodeMouseEventArgs evtArgs)
		{
			base.OnDoubleClick (evtArgs);
				if(GridCellDoubleClick != null)
					GridCellDoubleClick(this,evtArgs);
		}*/

        /*public override void Draw(Graphics grfx)
        {
            StringFormat fmt = this.GetStringFormat();
            if (this.Visible)
            {
                Matrix worldXform;
                if (this.Container != null)
                {
                    worldXform = this.Container.GetLabelTransform(this.matrix);
                }
                else
                {
                    worldXform = this.matrix;
                }

                RectangleF bounds = this.GetTextBox();
                Matrix prevXform = grfx.Transform;

                grfx.Transform = worldXform;
                grfx.MultiplyTransform(Global.ViewMatrix, MatrixOrder.Append);

                Brush backgroundBrush = this.BackgroundStyle.CreateBrush();
                grfx.FillRectangle(backgroundBrush, bounds.Left, bounds.Top, bounds.Width, bounds.Height);
                backgroundBrush.Dispose();
					 
                Font font = this.FontStyle.CreateFont();
                Brush fillBrush = this.FillStyle.CreateBrush();
                fmt.Trimming = StringTrimming.EllipsisCharacter;
                grfx.DrawString(this.Text, font, fillBrush, bounds, fmt);
                font.Dispose();
                fillBrush.Dispose();

                if (this.BorderStyle.ShowBorder)
                {
                    Pen pen = this.BorderStyle.CreatePen();
                    grfx.DrawRectangle(pen, bounds.Left, bounds.Top, bounds.Width, bounds.Height);
                    pen.Dispose();
                }

                grfx.Transform = prevXform;
            }
        }*/

		#endregion 
		
		# region Private Methods
		
		private void SetPropertyTable()
		{
			if(PT == null)
			{
				try
				{
					PropertyInfo[] properties =	this.GetType().GetProperties(BindingFlags.Instance | BindingFlags.DeclaredOnly | BindingFlags.Public);
				
					foreach	(PropertyInfo property in properties)
					{
						BrowsableAttribute[] attr = property.GetCustomAttributes(typeof(BrowsableAttribute),false) as BrowsableAttribute[];
						if(attr.Length > 0)
							if(((BrowsableAttribute)attr[0]).Browsable == false)
								continue;
						string propname	= property.Name;
						if(property.Name == "MasterPlantItem")
						{
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Lists of Master Plant Item" ,this.GetPropertyValue(property.Name),"",typeof(SymbolTypeConverter),this));
						}
						else if(property.Name == "PlantItem" )
						{
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Binding Information","Lists the Plant Item objects of Selected Master Plant Item",this.GetPropertyValue(property.Name),"",typeof(ObjectNameConverter),this));
						}
						else if(property.Name == "Link")
						{
							PT.Properties.Add(new PropertySpec(property.Name,property.PropertyType,"Link","Select the Link for the Symbol" ,this.GetPropertyValue(property.Name),"",typeof(LinkTypeConverter),this));
						}
						else
                        {
                            CategoryAttribute[] catAttr = property.GetCustomAttributes(typeof(CategoryAttribute), false) as CategoryAttribute[];
                            string category = "";
                            if (catAttr.Length > 0)
                                category = ((CategoryAttribute)catAttr[0]).Category;
                            else
                            {
                                category = "General	Properties";
                            }
							       TypeConverterAttribute[] typCnvtAttr = property.GetCustomAttributes(typeof(TypeConverterAttribute), false) as TypeConverterAttribute[];
                            string typeConverter = "";
                            if (typCnvtAttr.Length > 0)
                                typeConverter = ((TypeConverterAttribute)typCnvtAttr[0]).ConverterTypeName;
                            PT.Properties.Add(new PropertySpec(property.Name, property.PropertyType, category, "Sets the " + property.Name + " property of selected symbol", this.GetPropertyValue(property.Name), "", typeConverter, this));
						}
					}
				}	
				catch(Exception	ex)
				{
					Debug.Assert(false,ex.Message);
				}
			}
		}


		#endregion

		#region ISerializable Members

		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			try
			{
				base.GetObjectData (info, context);
				info.AddValue("_isBinded", this._isBinded);
				info.AddValue("_id", this._id);
				info.AddValue("_modeID", this._modeID);
				info.AddValue("_propertyID", this._propertyID);
				info.AddValue("_rowIndex", this._rowIndex);
				info.AddValue("_colIndex", this._colIndex);

				info.AddValue("_height", this._height);
				info.AddValue("_width", this._width);
				info.AddValue("_type", this._type);
				info.AddValue("_uomID", this._uomID);
				info.AddValue("_labelColorCode", this._labelColorCode);
				info.AddValue("_uomTypeID", this._uomTypeID);
				info.AddValue("_description", this._description);

				info.AddValue("_oldColSpan",this._oldColSpan);
				info.AddValue("_oldRowSpan",this._oldRowSpan);
                info.AddValue("_colSpan", this._colSpan);
                info.AddValue("_rowSpan", this._rowSpan);

			}
			catch(Exception ex)
			{
			}
		}

		#endregion

	}
}
